// modified Stream class, copied OFstream.H
// data can now be attached to an existing file
// To append lines to file


#ifndef VFstream_H
#define VFstream_H

#include "OFstream.H"

// * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * //

namespace Foam
{

class VFstream;

/*---------------------------------------------------------------------------*\
                       Class OFstreamAllocator Declaration
\*---------------------------------------------------------------------------*/

class VFstreamAllocator //:public OFstreamAllocator
{
    friend class VFstream;

    ostream* ofPtr_;

    // Constructors

        //- Construct from pathname
        VFstreamAllocator
        (
            const fileName& pathname,
            ios_base::openmode mode,
            IOstream::compressionType compression
        );

    // Destructor

        ~VFstreamAllocator();
};


/*---------------------------------------------------------------------------*\
                           Class OFstream Declaration
\*---------------------------------------------------------------------------*/

class VFstream
:
    private VFstreamAllocator,
    public OSstream
{
        fileName pathname_;
public:

    // Declare name of the class and it's debug switch
    ClassName("VFstream");


    // Constructors

        //- Construct from pathname
        VFstream
        (
            const fileName& pathname,
            ios_base::openmode mode = ios_base::out|ios_base::trunc,
            streamFormat format=ASCII,
            versionNumber version=currentVersion,
            compressionType compression=UNCOMPRESSED
        );


    // Destructor

        ~VFstream();
};
} // End namespace Foam
#endif

#include "OSspecific.H"
#include "gzstream.h"


Foam::VFstreamAllocator::VFstreamAllocator
(
    const fileName& pathname,
    ios_base::openmode mode,
    IOstream::compressionType compression
)
:
    ofPtr_(NULL)
{
    if (pathname.empty())
    {
        if (OFstream::debug)
        {
            Info<< "OFstreamAllocator::OFstreamAllocator(const fileName&) : "
                   "cannot open null file " << endl;
        }
    }

    if (compression == IOstream::COMPRESSED)
    {
        // get identically named uncompressed version out of the way
        if (isFile(pathname, false))
        {
            rm(pathname);
        }

        ofPtr_ = new ogzstream((pathname + ".gz").c_str(), mode);
    }
    else
    {
        // get identically named compressed version out of the way
        if (isFile(pathname + ".gz", false))
        {
            rm(pathname + ".gz");
        }

        ofPtr_ = new ofstream(pathname.c_str(), mode);
    }
}


Foam::VFstreamAllocator::~VFstreamAllocator()
{
    delete ofPtr_;
}


Foam::VFstream::VFstream
(
    const fileName& pathname,
    ios_base::openmode mode,
    streamFormat format,
    versionNumber version,
    compressionType compression
)
:
    VFstreamAllocator(pathname, mode, compression),
    OSstream(*ofPtr_, "OFstream.sinkFile_", format, version, compression),
    pathname_(pathname)
{
  setClosed();
  setState(ofPtr_->rdstate());
  
  if (!good())
    {
      setBad();
    }
  else
    {
      setOpened();
    }
  
  lineNumber_ = 1;
}


// * * * * * * * * * * * * * * * * Destructor  * * * * * * * * * * * * * * * //

Foam::VFstream::~VFstream()
{}
