/*---------------------------------------------------------------------------*\
  =========                 |
  \\      /  F ield         | OpenFOAM: The Open Source CFD Toolbox
   \\    /   O peration     |
    \\  /    A nd           | Copyright (C) 2011-2016 OpenFOAM Foundation
     \\/     M anipulation  |
-------------------------------------------------------------------------------
License
    This file is part of OpenFOAM.

    OpenFOAM is free software: you can redistribute it and/or modify it
    under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    OpenFOAM is distributed in the hope that it will be useful, but WITHOUT
    ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
    FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
    for more details.

    You should have received a copy of the GNU General Public License
    along with OpenFOAM.  If not, see <http://www.gnu.org/licenses/>.

Class
    Foam::fixedFluxMassCFvPatchScalarField

Group
    grpInletBoundaryConditions grpWallBoundaryConditions

Description
    This boundary condition sets the Mass gradient to the provided value
    such that the flux on the boundary is that specified by the velocity
    boundary condition.

    Example of the boundary condition specification:
    \verbatim
    <patchName>
    {
        type            fixedFluxMassC;
    }
    \endverbatim

See also
    Foam::fixedGradientFvPatchField

SourceFiles
    fixedFluxMassCFvPatchScalarField.C

\*---------------------------------------------------------------------------*/

#ifndef fixedFluxMassCFvPatchScalarFields_H
#define fixedFluxMassCFvPatchScalarFields_H

#include "fvPatchFields.H"
#include "fixedGradientFvPatchFields.H"

// * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * //

namespace Foam
{

/*---------------------------------------------------------------------------*\
             Class fixedFluxMassCFvPatchScalarField Declaration
\*---------------------------------------------------------------------------*/

class fixedFluxMassCFvPatchScalarField
:
    public fixedGradientFvPatchScalarField
{
    // Private data
        scalar currentDensity_;
        scalar molarMassA_;
        scalar valencyNumberOfIons_;
        scalar massDiffusivityAB_;

public:

    //- Runtime type information
    TypeName("fixedFluxMassC");


    // Constructors

        //- Construct from patch and internal field
        fixedFluxMassCFvPatchScalarField
        (
            const fvPatch&,
            const DimensionedField<scalar, volMesh>&
        );

        //- Construct from patch, internal field and dictionary
        fixedFluxMassCFvPatchScalarField
        (
            const fvPatch&,
            const DimensionedField<scalar, volMesh>&,
            const dictionary&
        );

        //- Construct by mapping given fixedFluxMassCFvPatchScalarField onto
        //  a new patch
        fixedFluxMassCFvPatchScalarField
        (
            const fixedFluxMassCFvPatchScalarField&,
            const fvPatch&,
            const DimensionedField<scalar, volMesh>&,
            const fvPatchFieldMapper&
        );

        //- Construct as copy
        fixedFluxMassCFvPatchScalarField
        (
            const fixedFluxMassCFvPatchScalarField&
        );

        //- Construct and return a clone
        virtual tmp<fvPatchScalarField> clone() const
        {
            return tmp<fvPatchScalarField>
            (
                new fixedFluxMassCFvPatchScalarField(*this)
            );
        }

        //- Construct as copy setting internal field reference
        fixedFluxMassCFvPatchScalarField
        (
            const fixedFluxMassCFvPatchScalarField&,
            const DimensionedField<scalar, volMesh>&
        );

        //- Construct and return a clone setting internal field reference
        virtual tmp<fvPatchScalarField> clone
        (
            const DimensionedField<scalar, volMesh>& iF
        ) const
        {
            return tmp<fvPatchScalarField>
            (
                new fixedFluxMassCFvPatchScalarField(*this, iF)
            );
        }


    // Member functions
        scalar& currentDensity()
            {
                return currentDensity_;
            }
        scalar& molarMassA()
            {
                return molarMassA_;
            }
        scalar& valencyNumberOfIons()
            {
                return valencyNumberOfIons_;
            }
        scalar& massDiffusityAB()
            {
                return massDiffusivityAB_;
            }
        //- Update the patch Mass gradient field
        virtual void updateCoeffs();

        //- Write
        virtual void write(Ostream&) const;
};


// * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * //

} // End namespace Foam

// * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * //

#include "volFields.H"

// * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * //

#endif

// ************************************************************************* //
