/*---------------------------------------------------------------------------*\
  =========                 |
  \\      /  F ield         | OpenFOAM: The Open Source CFD Toolbox
   \\    /   O peration     |
    \\  /    A nd           | Copyright (C) 2011-2016 OpenFOAM Foundation
     \\/     M anipulation  |
-------------------------------------------------------------------------------
License
    This file is part of OpenFOAM.

    OpenFOAM is free software: you can redistribute it and/or modify it
    under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    OpenFOAM is distributed in the hope that it will be useful, but WITHOUT
    ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
    FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
    for more details.

    You should have received a copy of the GNU General Public License
    along with OpenFOAM.  If not, see <http://www.gnu.org/licenses/>.

Application
    hmctFoam

Description
    Heat and mass trasport in single region.
    Solves the mass and heat transport in the hypothesis of pure diffusion.
    Single region solver.

\*---------------------------------------------------------------------------*/

#include "fvCFD.H"
#include "fvOptions.H"
#include "pimpleControl.H"
#include "clusterControl.H"

// * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * //

int main(int argc, char *argv[])
{
    #include "setRootCase.H"
    #include "gitVersion.H"
    #include "createTime.H"
    #include "createMesh.H"
    #include "createControl.H"
    #include "createSolverParameters.H"
    #include "createFvOptions.H"

    // * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * //

    const bool massConcentration = runTime.controlDict().lookupOrDefault<bool>(
    "massConcentration", false
    );
    const bool thermal = runTime.controlDict().lookupOrDefault<bool>(
    "thermal", false
    );

    #include "createHMFields.H"

        while (runTime.run())
        {
            runTime++;
            Info<< "Time = " << runTime.timeName() << nl << endl;
            while (pimple.correctNonOrthogonal())
            {
                rhok = 1;
                if(massConcentration){
                #include "solveMassDiffEqn.H"
                }
                if(thermal){
                #include "solveTDiffEqn.H"
                }
            }
            #include "write.H"
        }

    Info<< "End\n" << endl;

    return 0;
}
// ************************************************************************* //
