#ifndef deflatedPCG_H
#define deflatedPCG_H

#include "lduMatrix.H"
#include "fvCFD.H"
#include "solution.H"

// * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * //

namespace Foam
{

/*---------------------------------------------------------------------------*\
                           Class deflatedPCG Declaration
\*---------------------------------------------------------------------------*/

class deflatedPCG
:
    public lduMatrix::solver
{
    // Private Member Functions

        //- Disallow default bitwise copy construct
        deflatedPCG(const deflatedPCG&);

        //- Disallow default bitwise assignment
        void operator=(const deflatedPCG&);


public:

    //- Runtime type information
    TypeName("deflatedPCG");


    // Constructors

        //- Construct from matrix components and solver controls
        deflatedPCG
        (
            const word& fieldName,
            const lduMatrix& matrix,
            const FieldField<Field, scalar>& interfaceBouCoeffs,
            const FieldField<Field, scalar>& interfaceIntCoeffs,
            const lduInterfaceFieldPtrsList& interfaces,
            const dictionary& solverControls
        );


    //- Destructor
    virtual ~deflatedPCG()
    {}


    // Member Functions

        //- Solve the matrix with this solver
        virtual solverPerformance solve
        (
            scalarField& psi,
            const scalarField& source,
            const direction cmpt=0
        ) const;
};

bool checkDeflated(volScalarField& p)
{
    solution fvSolution(p.db(),"fvSolution");

    dictionary solver = fvSolution.solverDict(p.name());
    word solverName = solver.lookup("solver");
    bool deflated = false;
    if(solverName == "deflatedPCG")
    {
        deflated = true;
        if(!p.needReference())
	{
            FatalErrorIn("checkDeflated")
                    << "deflated solver can only be used for pure Neumann boundary conditions"
                    << exit(FatalError);
	}

        Info << "Using deflated solver for " << p.name() << endl;
    }

    return deflated;
}

} // End namespace Foam

// * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * //

#endif
