/*---------------------------------------------------------------------------*\
  =========                 |
  \\      /  F ield         | OpenFOAM: The Open Source CFD Toolbox
   \\    /   O peration     | Website:  https://openfoam.org
    \\  /    A nd           | Copyright (C) 2019 OpenFOAM Foundation
     \\/     M anipulation  |
-------------------------------------------------------------------------------
License
    This file is part of OpenFOAM.

    OpenFOAM is free software: you can redistribute it and/or modify it
    under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    OpenFOAM is distributed in the hope that it will be useful, but WITHOUT
    ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
    FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
    for more details.

    You should have received a copy of the GNU General Public License
    along with OpenFOAM.  If not, see <http://www.gnu.org/licenses/>.

\*---------------------------------------------------------------------------*/

#include "spaceVariableFluxMassCFvPatchScalarField.H"
#include "fvPatchFieldMapper.H"
#include "volFields.H"
#include "surfaceFields.H"
#include "addToRunTimeSelectionTable.H"
#include "physicoChemicalConstants.H"
// * * * * * * * * * * * * * Private Member Functions  * * * * * * * * * * * //

// * * * * * * * * * * * * * * * * Constructors  * * * * * * * * * * * * * * //

Foam::spaceVariableFluxMassFvPatchScalarField::
spaceVariableFluxMassFvPatchScalarField
(
    const fvPatch& p,
    const DimensionedField<scalar, volMesh>& iF
)
:
    fixedGradientFvPatchScalarField(p, iF),
    meanCurrentDensity_(0.0),
    value0_(0.0),
    value1_(0.0),
    value2_(0.0),
    value3_(0.0),
    value4_(0.0),
    value5_(0.0),
    x_(1, 0, 0)
{
}

Foam::spaceVariableFluxMassFvPatchScalarField::
spaceVariableFluxMassFvPatchScalarField
(
    const fvPatch& p,
    const DimensionedField<scalar, volMesh>& iF,
    const dictionary& dict
)
:
    fixedGradientFvPatchScalarField(p, iF),
    meanCurrentDensity_(readScalar(dict.lookup("meanCurrentDensity"))),
    value0_(readScalar(dict.lookup("value0"))),
    value1_(readScalar(dict.lookup("value1"))),
    value2_(readScalar(dict.lookup("value2"))),
    value3_(readScalar(dict.lookup("value3"))),
    value4_(readScalar(dict.lookup("value4"))),
    value5_(readScalar(dict.lookup("value5"))),
    x_(dict.lookup("x"))
{
    if (dict.found("value") && dict.found("gradient"))
    {
       fvPatchField<scalar>::operator=
       (
	   scalarField("value", dict, p.size())
       );
       gradient() = scalarField("gradient", dict, p.size());
    }
    else
    {
	 fvPatchField<scalar>::operator=(patchInternalField());
	 gradient() = 0.0;
    }
}


Foam::spaceVariableFluxMassFvPatchScalarField::
spaceVariableFluxMassFvPatchScalarField
(
    const spaceVariableFluxMassFvPatchScalarField& ptf,
    const fvPatch& p,
    const DimensionedField<scalar, volMesh>& iF,
    const fvPatchFieldMapper& mapper
)
:
    fixedGradientFvPatchScalarField(p, iF),
    meanCurrentDensity_(ptf.meanCurrentDensity_),
    value0_(ptf.value0_),
    value1_(ptf.value1_),
    value2_(ptf.value2_),
    value3_(ptf.value3_),
    value4_(ptf.value4_),
    value5_(ptf.value5_),
    x_(ptf.x_)
{
    patchType() = ptf.patchType();
    gradient() = 0.0;
    gradient().map(ptf.gradient(), mapper);
    if (notNull(iF) && iF.size())
    {
	scalarField::operator=
	(
	  patchInternalField() + gradient()*(patch().nf() & patch().delta())
	);
    }
    else
    {
	this->map(ptf, mapper);
    }
}

Foam::spaceVariableFluxMassFvPatchScalarField::
spaceVariableFluxMassFvPatchScalarField
(
    const spaceVariableFluxMassFvPatchScalarField& wbppsf
)
:
    fixedGradientFvPatchScalarField(wbppsf),
    meanCurrentDensity_(wbppsf.meanCurrentDensity_),
    value0_(wbppsf.value0_),
    value1_(wbppsf.value1_),
    value2_(wbppsf.value2_),
    value3_(wbppsf.value3_),
    value4_(wbppsf.value4_),
    value5_(wbppsf.value5_),
    x_(wbppsf.x_)
{}


Foam::spaceVariableFluxMassFvPatchScalarField::
spaceVariableFluxMassFvPatchScalarField
(
    const spaceVariableFluxMassFvPatchScalarField& wbppsf,
    const DimensionedField<scalar, volMesh>& iF
)
:
    fixedGradientFvPatchScalarField(wbppsf, iF),
    meanCurrentDensity_(wbppsf.meanCurrentDensity_),
    value0_(wbppsf.value0_),
    value1_(wbppsf.value1_),
    value2_(wbppsf.value2_),
    value3_(wbppsf.value3_),
    value4_(wbppsf.value4_),
    value5_(wbppsf.value5_),
    x_(wbppsf.x_)
{}



// * * * * * * * * * * * * * * * Member Functions  * * * * * * * * * * * * * //

void Foam::spaceVariableFluxMassFvPatchScalarField::updateCoeffs()
{
    if (updated())
    {
	return;
    }
    const dictionary& transportProperties = this->db().lookupObject<IOdictionary>
    (
		 "transportProperties"
    );
    dimensionedScalar M_A
    (
    "M_A",
    dimensionSet(1,0,0,0,-1,0,0),
    transportProperties.lookup("M_A")
    );
    dimensionedScalar D_AB
    (
    "D_AB",
    dimensionSet(0,2,-1,0,0,0,0),
    transportProperties.lookup("D_AB")
    );
    dimensionedScalar z_A
    (
    "z_A",
    dimensionSet(1,0,0,0,-1,1,0),
    transportProperties.lookup("z_A")
    );
    boundBox bb(patch().patch().localPoints(), true);
    vector ctr = (bb.min());
    const vectorField& c = patch().Cf();
    scalarField coord = ((c - ctr) & x_);
    scalarField currentDensity = ((meanCurrentDensity_) *
    (value0_ +
     value1_ * coord +
     value2_* sqr(coord) +
     value3_ * pow3(coord) +
     value4_ * pow4(coord) +
     value5_ * pow5(coord)));
    gradient() = ((currentDensity* M_A.value())/
    (constant::physicoChemical::F.value() * D_AB.value() * z_A.value()));
    fixedGradientFvPatchScalarField::updateCoeffs();
}


void Foam::spaceVariableFluxMassFvPatchScalarField::write
(
    Ostream& os
) const
{
    fixedGradientFvPatchScalarField::write(os);
    os.writeKeyword("meanCurrentDensity") << meanCurrentDensity_ << token::END_STATEMENT << nl;
    os.writeKeyword("value0") << value0_ << token::END_STATEMENT << nl;
    os.writeKeyword("value1") << value1_ << token::END_STATEMENT << nl;
    os.writeKeyword("value2") << value2_ << token::END_STATEMENT << nl;
    os.writeKeyword("value3") << value3_ << token::END_STATEMENT << nl;
    os.writeKeyword("value4") << value4_ << token::END_STATEMENT << nl;
    os.writeKeyword("value5") << value5_ << token::END_STATEMENT << nl;
    os.writeKeyword("x") << x_ << token::END_STATEMENT << nl;
    writeEntry("value", os);
}


// * * * * * * * * * * * * * * Build Macro Function  * * * * * * * * * * * * //

namespace Foam
{
    makePatchTypeField
    (
	fvPatchScalarField,
	spaceVariableFluxMassFvPatchScalarField
    );
}

// ************************************************************************* //
