# Script for an axisymettric complex geometry
file = open("./NEW/system/blockMeshDict", "w")
import math as m


def xc(r):
    # return the x position at the specified radius.
    return r * m.cos(gamma * m.pi / 180.0)


def yp(r):
    # return the positive y position at the specified radius.
    return r * m.sin(gamma * m.pi / 180.0)


def yn(r):
    # return the negative y position at the specified radius.
    return -r * m.sin(gamma * m.pi / 180.0)


def pvert(x, y, z, n):
    # file.write a vertex position for blockMeshDict.
    file.write("  (%g %g %g)\t// vertex %d \n" % (x, y, z, n))


# all measures are in mm and deg
gamma = 2.5  # half of the angle of the define
rr = R_R
h_P = HH_P

file.write(
    """/*--------------------------------*- C++ -*----------------------------------*\\
| =========                 |                                                 |
| \\\      /  F ield         | OpenFOAM: The Open Source CFD Toolbox           |
|  \\\    /   O peration     | Version:  6.0                                   |
|   \\\  /    A nd           | Web:      www.OpenFOAM.org                      |
|    \\\/     M anipulation  |                                                 |
\*---------------------------------------------------------------------------*/
FoamFile
{
    version     2.0;
    format      ascii;
    class       dictionary;
    object      blockMeshDict;
}
// * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * //

convertToMeters 0.001;	// From mm to m
nLr NN_R;			// cells on x - Lateral direction
nLy 1;		    // cells on y - Lateral direction
nPz NN_Z;			// cells on z - Positive electrode

vertices
("""
)
pvert(0, 0, 0, 0)
pvert(xc(rr), yp(rr), 0, 1)
pvert(xc(rr), yp(rr), h_P, 2)
pvert(0, 0, h_P, 3)
pvert(xc(rr), yn(rr), 0, 4)
pvert(xc(rr), yn(rr), h_P, 5)

file.write(
    """);

blocks
("""
)

file.write(
    """  hex (0 4 1 0 3 5 2 3)       ($nLr $nLy $nPz)
   simpleGrading (
   (
   (0.2 0.25 2.0)
   (0.30 0.250 1.5)
   (0.30 0.250 0.75)
   (0.2 0.25 0.5)
   )
   (1)
   (
   (0.35 0.400 1.5)
   (0.30 0.200 1.0)
   (0.35 0.400 0.75)
   )
   )
);"""
)
file.write(
    """
edges
();"""
)
file.write(
    """

boundary
(
     ypFace
     {
           type wedge;
           faces
           (
               (0 3 2 1)
           );
      }
     ymFace
     {
           type wedge;
           faces
           (
               (0 4 5 3)
           );
      }
     rrFace
     {
           type wall;
           faces
           (
               (1 2 5 4)
           );
      }
     zmFace
     {
           type wall;
           faces
           (
               (0 1 4 0)
           );
      }
     zpFace
     {
           type wall;
           faces
           (
               (3 5 2 3)
           );
      }
     axis
     {
           type empty;
           faces
           (
               (3 0 0 3)
           );
      }
);

mergePatchPairs
(
);"""
)
